@@ -63,6 +63,116 @@ use constant {
     DIFFERENCE_PATH => '/lists/diff/json',
 };
 
+
+=head2 get_list( $name ) -> List
+
+Get the list with the given name, if it exists. Returns undef
+if the given list is not accessible (either because it does not
+exist or because the service is not authenticated to the correct
+user account).
+
+=head2 get_lists() -> list[List]
+
+Get all the lists that this service has access to.
+
+=head2 list_names() -> list[Str]
+
+Get all the names of the lists this service has access to.
+
+=head2 new_list(content => $content, [type => $class], [name => $name], [description => $description])
+
+Make a new list on the server with the given content and return
+a L<Webservice::InterMine::List> object referring to it.
+
+Args:
+
+=over 2
+
+=item * content
+
+can be a filename, an array ref of identifiers, a string 
+containing identifiers (separated by whitespace and quoted, if
+necessary by double-quotes), or a L<Webservice::InterMine::Query>
+(specifically, a L<Webservice::InterMine::Query::Roles::Listable>
+query.
+
+=item * type (required, unless the content is a query)
+
+The type of the items in this list. The type must be one of the
+of the classes in the model.
+
+=item * name [optional]
+
+The name of the list to create. If not name is supplied, one 
+will be generated for you, and the resulting list will be cleaned
+up (deleted) when the program exits. (Renaming the list later
+will prevent this clean up, as will setting the $CLEAN_UP variable
+in L<Webservice::InterMine>.)
+
+=item * description [optional]
+
+A long form description of the nature of this list. Optional. One will 
+be generated if none is provided.
+
+=item * path [required if the content is a Template]
+
+The path to determine the list type for Templates.
+
+=back
+
+=head2 union(\@lists, [$name, $description)
+
+Join the given lists to create a new one (with the optional name
+and description) from their union. The lists can be given as
+L<Webservice::InterMine::List> objects,
+L<Webservice::InterMine::Query::Roles::Listable> queries,
+or the names of the lists to join.
+
+=head2 subtract(\@from, \@to_be_subtracted, [$name, $description])
+
+Subtract the lists to be subtracted from the union of the
+list of lists to subtract from, creating a new one 
+(with the optional name and description). The lists can be given as
+L<Webservice::InterMine::List> objects,
+L<Webservice::InterMine::Query::Roles::Listable> queries,
+or the names of lists the service has access to.
+
+=head2 intersect(\@lists, [$name, $description)
+
+Intersect the given lists to create a new one (with the optional name
+and description) from their intersection. The lists can be given as
+L<Webservice::InterMine::List> objects,
+L<Webservice::InterMine::Query::Roles::Listable> queries,
+or the names of lists the service has access to.
+
+=head2 symmetric_difference(\@lists, [$name, $description)
+
+Calculate the symmetric difference of the given lists 
+to create a new one (with the optional name
+and description). The lists can be given as
+L<Webservice::InterMine::List> objects,
+L<Webservice::InterMine::Query::Roles::Listable> queries,
+or the names of lists the service has access to.
+
+=head2 delete_lists(@lists)
+
+Delete the given lists from the service. If the lists don't exist,
+an error will be thrown. Consider also calling the 
+C<delete> method directly on L<Webservice::InterMine::List>
+objects.
+
+=head2 delete_temp_lists()
+
+Delete all anonymous list created by the programme. This method
+is automatically called on programme exist unless the variable 
+C<$Webservice::InterMine::CLEAN_UP> is set to a false value.
+
+=head2 list_count -> Int
+
+Get the number of lists in the service at present
+
+=cut
+
 sub BUILD {
     my $self = shift;
     $self->refresh_lists;
@@ -141,6 +251,14 @@ sub get_lists_with_object {
     }
 }
 
+=head2 refresh_lists
+
+Make sure the list information this object has access to is
+up to date by checking for updates from the server. This method
+is called when any changes are made to lists.
+
+=cut
+
 sub refresh_lists {
     my $self = shift;
     my $latest_data = $self->service->get_list_data;
