@@ -9,7 +9,8 @@ with(
 
 use Carp;
 use List::Util qw/reduce/;
-use List::MoreUtils qw/uniq/;
+use List::MoreUtils qw/uniq natatime/;
+use Scalar::Util qw/blessed/;
 
 use MooseX::Types::Moose qw/Str Bool/;
 use InterMine::Model::Types qw/PathList PathHash PathString/;
@@ -34,19 +35,44 @@ has '+name' => (
     coerce => 1,
 );
 
+=head2 root_path - The root of this query
+
+The starting table for this query.
+
+=head2 has_root_path - Whether or not this query has a root set
+
+=cut
+
 has root_path => (
     init_arg  => 'class', 
     isa       => PathString, 
     is        => 'ro',
+    writer    => '_set_root',
     coerce    => 1,
     predicate => 'has_root_path',
     trigger    => sub {
         my ($self, $root) = @_;
-        my $err = validate_path( $self->model, $root, $self->type_dict );
-        confess $err if $err;
+        if ($self->is_validating) {
+            my $err = validate_path( $self->model, $root, $self->type_dict );
+            confess $err if $err;
+        }
     },
 );
 
+=head2 has_sort_order - whether or not this query has a defined sort-order
+
+=head2 push_sort_order - Add a sort order element to the sort order
+
+=head2 sort_orders - get the list of sort orders
+
+=head2 joined_so($sep) - Join the sort orders with a separator
+
+=head2 clear_sort_order - delete all sort order information from the query
+
+=head2 sort_order_is_empty - whether or not there are any elements in the sort order list
+
+=cut
+
 has _sort_order => (
     traits     => ['Array'],
     is         => 'ro',
@@ -68,24 +94,9 @@ has _sort_order => (
     },
 );
 
-sub prefix_pathfeature {
-    my ($self, $pf) = @_;
-    if ($self->has_root_path) {
-        my $root = $self->root_path;
-        unless ($pf->path =~ /^$root/) {
-            my $new_path = $self->root_path . '.' . $pf->path;
-            $pf->set_path($new_path);
-        }
-    }
-}
+=head2 sort_order - Return the string representing the sort order
 
-sub add_sort_order {
-    my $self = shift;
-    my @args = @_;
-    my $so = Webservice::InterMine::SortOrder->new(@args);
-    $self->prefix_pathfeature($so);
-    $self->push_sort_order($so);
-}
+=cut
 
 sub sort_order {
     my $self = shift;
@@ -98,6 +109,72 @@ sub sort_order {
     } 
 }
 
+=head2 add_sort_order(@args) - add a sort order element to this query
+
+=cut
+
+sub add_sort_order {
+    my $self = shift;
+    my @args = @_;
+    my $so = Webservice::InterMine::SortOrder->new(@args);
+    $self->prefix_pathfeature($so);
+    $self->push_sort_order($so);
+}
+
+=head2 order_by(@args) -> $self
+
+Replace any existing sort order by the one defined with the given arguments. 
+Return self to allow method chaining.
+
+=cut
+
+sub order_by {
+    my ($self, @args) = @_;
+    $self->clear_sort_order;
+    my $it = natatime(2, @args);
+    while (my @pair = $it->()) {
+        $self->add_sort_order(@pair);
+    }
+    return $self;
+}
+
+=head2 prefix_pathfeature - prefix the path of a PathFeature with the query's root.
+
+Used internally to process shortened, headless paths.
+
+=cut
+
+sub prefix_pathfeature {
+    my ($self, $pf) = @_;
+    my $new_path = $self->prefix_path($pf->path);
+    $pf->set_path($new_path) unless ($new_path eq $pf->path);
+}
+
+=head2 prefix_path($path) -> a prefixed path
+
+Used internally to process shortened, headless paths.
+
+=cut
+
+sub prefix_path {
+    my ($self, $str) = @_;
+    if ($self->has_root_path) {
+        my $root = $self->root_path;
+        my $new_path = ($str =~ /^$root/) ? $str : $self->root_path . ".$str";
+        return $new_path;
+    } else {
+        my ($root) = split(/\./, $str);
+        $self->_set_root($root);
+        return $str;
+    }
+}
+
+=head2 DEMOLISH
+
+Called upon object destruction
+
+=cut
+
 sub DEMOLISH {
     my $self = shift;
     $self->suspend_validation;
@@ -108,6 +185,10 @@ sub _build__sort_order {
     return $self->get_view(0);
 }
 
+=head2 set_sort_order(@elements) - replace any existing order with the given ones.
+
+=cut
+
 sub set_sort_order {
     my $self = shift;
     $self->_set_sort_order( join( ' ', @_ ) );
@@ -132,17 +213,60 @@ has view => (
     },
 );
 
-sub add_views {
-    my $self = shift;
-    return $self->add_view(@_);
-}
+=head2 view -> ArrayRef
+
+Get the view as an array-ref
+
+=head2 views -> list
+
+Get the paths that make up the output columns (the view)
+
+=head2 get_view($index) -> path
+
+Get the view at the specified index
+
+=head2 joined_view($sep) -> Str
+
+Get a string consisting of the view paths joined with the given separator.
+
+=head2 view_is_empty -> Bool
+
+Return true if the view is currently empty.
+
+=head2 clear_view
+
+Clear the current view
+
+=head2 view_size -> Int
+
+Get the number of output columns
+
+=head2 add_view
+
+alias for add_views
+
+=head2 select
+
+alias for add_view
+
+=head2 add_views(@views)
+
+Add the given views to the view list, first preprending
+the query's root, and checking for validity.
+
+=cut
 
 sub add_view {
+    goto &add_views;
+}
+
+sub select {
+    goto &add_views;
+}
+
+sub add_views {
     my $self = shift;
-    my @views = map {split} @_;
-    if (my $root = $self->root_path) {
-        @views = map {(/^$root/) ? $_ : "$root.$_"} @views;
-    }
+    my @views = map {$self->prefix_path($_)} map {split} @_;
     my @expanded_views;
     for my $view (@views) {
         if ($view =~ /(.*)\.\*$/) {
@@ -155,6 +279,7 @@ sub add_view {
         }
     }
     $self->_add_views(@expanded_views);
+    return $self;
 }
 
 after qr/^add_/ => sub {
@@ -162,6 +287,41 @@ after qr/^add_/ => sub {
     $self->_validate;
 };
 
+=head2 constraints
+
+Get the list of constraints in the order they were added. 
+Returns an arrayref in scalar context,
+and a list in list context.
+
+=head2 all_constraints
+
+Get all constraints as a list, in the order they were added.
+
+=head2 map_constraints($coderef) -> list
+
+Apply the coderef to each constraint in turn (ala C<map>)
+and return the result of each call.
+
+=head2 find_constraints($coderef) -> list
+
+Apply the coderef to each constraint in turn (ala C<grep>) 
+and return the constraints for which the code returns
+a truthy value.
+
+=head2 delete_constraint($index)
+
+Delete the constraint with the given index from the list.
+
+=head2 count_constraints -> Int
+
+Get the number of constraints on this query.
+
+=head2 clear_constraints 
+
+Remove all constraints from this query
+
+=cut
+
 has constraints => (
     traits     => ['Array'],
     is         => 'ro',
@@ -170,10 +330,11 @@ has constraints => (
     auto_deref => 1,
     handles    => {
         all_constraints   => 'elements',
-        push_constraint   => 'push',
+        _push_constraint   => 'push',
         find_constraints  => 'grep',
         map_constraints   => 'map',
         delete_constraint => 'delete',
+        _get_constraint   => 'get',
         count_constraints => 'count',
         clear_constraints => 'clear',
     },
@@ -199,6 +360,14 @@ sub get_constraint {
     return $matches[0];
 }
 
+=head2 remove($constraint)
+
+Remove the given object from the query, where the object is
+a constraint child object (a path-description, a constraint or 
+a join).
+
+=cut
+
 sub remove {
     my $self     = shift;
     my $delendum = shift;    # Constraintum delendum est
@@ -220,6 +389,14 @@ sub remove {
     }
 }
 
+=head2 coded_constraints -> list|Int
+
+Return the constraints that have codes and can participate in logic. 
+Returns a list in list context and the size of the list in scalar
+context.
+
+=cut
+
 sub coded_constraints {
     my $self      = shift;
     my $criterion = sub {
@@ -228,6 +405,14 @@ sub coded_constraints {
     return $self->find_constraints($criterion);
 }
 
+=head2 sub_class_constraints -> list|Int
+
+Return the constraints that constrain object types, and cannot participate in logic. 
+Returns a list in list context and the size of the list in scalar
+context.
+
+=cut
+
 sub sub_class_constraints {
     my $self = shift;
     my $criterion =
@@ -235,16 +420,39 @@ sub sub_class_constraints {
     return $self->find_constraints($criterion);
 }
 
+=head2 constraint_codes -> list
+
+Return the codes (single characters from 'A' to 'Z') that this query
+uses.
+
+=cut
+
 sub constraint_codes {
     my $self = shift;
     return map { $_->code } $self->coded_constraints;
 }
 
-after push_constraint => sub {
+after _push_constraint => sub {
     my $self = shift;
-    $self->clear_logic;
+    if ($self->coded_constraints < 2 or not $self->has_logic) {
+        return;
+    }
+    my $new_constraint = $self->_get_constraint(-1);
+    unless ($new_constraint->isa('Webservice::InterMine::Constraint::SubClass') 
+            or $self->count_constraints <= 1) {
+        my $old_logic = $self->logic;
+        $self->set_logic($old_logic & $new_constraint);
+    }
 };
 
+=head2 type_dict -> hashref
+
+returns a hashref with the mapping from class => subclass
+for all constrained types within the query. This summarises the
+information from subclass constraints.
+
+=cut
+
 sub type_dict {
     my $self = shift;
     my @sccs = $self->sub_class_constraints;
@@ -255,12 +463,43 @@ sub type_dict {
     return {%type_dict};
 }
 
+=head2 subclasses -> list
+
+Return the list of subclasses as constrained in the query.
+
+=cut
+
 sub subclasses {
     my $self = shift;
     my @sccs = $self->sub_class_constraints;
     return map { $_->type } @sccs;
 }
 
+=head2 joins -> arrayref
+
+Return an arrayref of the L<Webservice::InterMine::Join> objects
+on this query, in the order they were added.
+
+=head2 all_joins -> list
+
+Returns the L<Webservice::InterMine::Join> objects of this query as a
+list in the order they were added to the query.
+
+=head2 map_joins($code) -> list
+
+Apply the codereference to each join in the query in turn and 
+return the results of the calls.
+
+=head2 clear_joins
+
+Remove all joins from the query
+
+=head2 delete_join($index)
+
+Remove the given join from the query.
+
+=cut
+
 has joins => (
     traits     => ['Array'],
     is         => 'ro',
@@ -269,14 +508,14 @@ has joins => (
     auto_deref => 1,
     handles    => {
         all_joins   => 'elements',
-        push_join   => 'push',
+        _push_join   => 'push',
         map_joins   => 'map',
         clear_joins => 'clear',
         delete_join => 'delete',
     }
 );
 
-=head2 add_join( $path )
+=head2 add_join( $path ) -> $self
 
 Specifies the join style of a path on the query. 
 The default join style this method adds is "OUTER", but
@@ -289,22 +528,29 @@ sub add_join {
     my $self = shift;
     my $join = Webservice::InterMine::Join->new(@_);
     $self->prefix_pathfeature($join);
-    $self->push_join($join);
+    $self->_push_join($join);
     return $self;
 }
 
+=head2 outerjoin($path) -> $self
+
+A shortcut for C<< add_join($path, 'OUTER') >>.
+
+=cut
+
+sub outerjoin {
+    my ($self, $path) = @_;
+    confess "Too many arguments - 1 expected" if (@_ > 2);
+    return $self->add_join($path, 'OUTER');
+}
+
 =head2 add_outer_join( $path )
 
-specify that this path is to be treated as an outer join.
+A shortcut for C<< add_join($path, 'OUTER') >>.
 
 =cut
 
-sub add_outer_join {
-    my $self = shift;
-    my $path = shift;
-    confess "Too many arguments to 'add_outer_join', 1 expected" if @_;
-    $self->add_join($path);
-}
+sub add_outer_join {goto &outerjoin}
 
 has path_descriptions => (
     traits     => ['Array'],
@@ -328,19 +574,57 @@ sub add_pathdescription {
     $self->push_path_description($pd);
     return $self;
 }
+
+=head2 logic -> Logic
+
+Returns the logic object for this query. This object stringifies to the
+logic expression used in the serialisation. 
+
+=head2 clear_logic 
+
+Remove the current logic from this query
+
+=head2 has_logic
+
+Return true if this query has logic set for it.
+
+=cut
+
 has logic => (
-    writer  => 'set_logic',
+    writer  => '_set_logic',
     reader  => 'logic',
     isa     => LogicOrStr,
     lazy    => 1,
     clearer => 'clear_logic',
-    trigger => \&check_logic,
+    predicate => 'has_logic',
+    trigger => \&_check_logic,
     default => sub {
         my $self = shift;
         reduce { $a & $b } $self->coded_constraints;
     },
 );
 
+=head2 set_logic($expr) -> self
+
+Sets the logic for this query, validating it in the process.
+Returns self to support chaining.
+
+  $query->set_logic("A or B and C");
+
+=cut
+
+sub set_logic {
+    my ($self, @args) = @_;
+    $self->_set_logic(@args);
+    return $self;
+}
+
+=head2 constraint_factory
+
+The object responsible for making constraints for this query.
+
+=cut
+
 has constraint_factory => (
     is         => 'ro',
     isa        => ConstraintFactory,
@@ -389,7 +673,7 @@ sub all_children {
 }
 ############### METHODS
 
-sub check_logic {
+sub _check_logic {
     my ( $self, $value ) = @_;
     unless ( blessed $value) {
         my $new_value = _parse_logic( $value, $self->coded_constraints );
@@ -451,6 +735,16 @@ sub _parse_logic {
     return eval join '', @processed_bits;
 }
 
+=head2 add_constraint(@args) -> constraint
+
+Adds a constraint corresponding to the given arguments, 
+and returns the new constraint.
+
+  $query->add_constraint('name', '=', 'foo');
+  $query->add_constraint(age => {lt => 50});
+
+=cut
+
 sub add_constraint {
     my $self       = shift;
     my %args       = parse_constraint_string(@_);
@@ -462,10 +756,77 @@ sub add_constraint {
         }
     }
     $self->prefix_pathfeature($constraint);
-    $self->push_constraint($constraint);
+    $self->_push_constraint($constraint);
     return $constraint;
 }
 
+=head2 search($constraints) -> self|results
+
+Add the given constraints to the query, where those constraints 
+are specified with an array-reference or a hash-reference. 
+In list context return the result of the query. In scalar context, return
+the query for chaining.
+
+This method is similar in interface to the method of the same name 
+in L<DBIx::Class::ResultSet>.
+
+=cut
+
+sub search {
+    my $self = shift;
+    my $constraints = shift;
+    my $results_args = shift || {as => 'jsonobjects', json => 'instantiate'};
+    if (ref $constraints eq 'HASH') {
+        while (my ($path, $con) = each(%$constraints)) {
+            $self->add_constraint($path, $con);
+        }
+    } elsif (ref $constraints eq 'ARRAY' and @$constraints % 2 == 0) {
+        for (my $i = 0; $i < (@$constraints - 1); $i += 2) {
+            my ($path, $con) = @{$constraints}[$i, ($i + 1)];
+            $self->add_constraint($path, $con);
+        }
+    } else {
+        carp "Bad argument: '$constraints' should be a hashref or an array ref with an even number of elements";
+    }
+
+    if (wantarray) {
+        return $self->results(%$results_args);
+    } else {
+        return $self;
+    }
+}
+
+=head2 where(@constraints) -> $self
+
+Add the given constraints to the query, and return self to support chaining
+
+  $query->where(
+    name     => "foo",
+    fullTime => 'true',
+    age      => {gt => 10},
+    'department.name'    => ['Sales', 'Accounting'],
+    'department.company' => {lookup => 'Foo'},
+  );
+
+=cut
+
+sub where {
+    my $self = shift;
+    if (@_ == 1) {
+        return $self->search(@_);
+    } else {
+        return $self->search([@_]);
+    }
+}
+
+=head2 parse_constraint_string
+
+Interpret the constraint arguments so that constraints
+can be constructed while allowing multiple representations of
+constraints to be understood.
+
+=cut
+
 sub parse_constraint_string {
     if ( @_ > 1 ) {
         if ( @_ % 2 == 0 ) {
@@ -477,6 +838,54 @@ sub parse_constraint_string {
                 return %args;
             }
         }
+        if ( @_ == 2 ) {
+            my ($path, $con) = @_;
+
+            if (not defined $con) {
+                return (path => $path, op => 'IS NULL');
+            }
+            unless (grep {$con eq $_} 'IS NULL', 'IS NOT NULL') {
+                my %args;
+                if (ref $con eq 'HASH') {
+                    %args = (path => $path, %$con);
+                } else {
+                    %args = (path => $path);
+                }
+
+                my $value = $con;
+
+                for my $key (keys %args) {
+                    unless (grep {$key eq $_} qw/path op code extra_value type/) {
+                        my $op = $key;
+                        $value = delete($args{$op});
+                        $args{op} = $op;
+                    }
+                }
+
+                if ($args{type}) {
+                    # Ignore
+                } elsif (not defined $value) {
+                    $args{op} = 'IS NULL'     if (grep {$args{op} eq $_} '=', 'eq', 'is');
+                    $args{op} = 'IS NOT NULL' if (grep {$args{op} eq $_} '!=', 'ne', 'isnt');
+                } elsif (ref $value eq 'ARRAY') {
+                    $args{op} ||= 'ONE OF';
+                    $args{values} = $value;
+                } elsif (blessed $value and $value->isa('Webservice::InterMine::List')) {
+                    $args{op} ||= 'IN';
+                    $args{value} = $value;
+                } elsif (blessed $value and $value->isa('InterMine::Model::Path')) {
+                    $args{op} ||= 'IS';
+                    $args{loop_path} = $value;
+                } elsif (blessed $value and $value->isa('InterMine::Model::ClassDescriptor')) {
+                    $args{type} ||= $value;
+                } else {
+                    $args{op} ||= '=';
+                    $args{value} = $value;
+                }
+                return %args;
+            }
+        }
+
         my %args;
         @args{qw/path op value extra_value/} = @_;
         if ( ref $args{value} eq 'ARRAY' ) {
@@ -497,7 +906,7 @@ sub parse_constraint_string {
                 (
                 IS\sNOT\sNULL|
                 IS\sNULL|
-                NOT\sIN|\S+
+                (?:NOT\s)?IN|\S+
                 )
         	   (
                ?:\s+(.*)
@@ -513,6 +922,12 @@ sub parse_constraint_string {
     }
 }
 
+=head2 clean_out_SCCs
+
+Remove pointless subclass constraints from the query.
+
+=cut
+
 sub clean_out_SCCs {
     my $self = shift;
     for ( $self->sub_class_constraints ) {
@@ -524,6 +939,12 @@ sub clean_out_SCCs {
     }
 }
 
+=head2 to_string 
+
+Return a readable representation of the query.
+
+=cut
+
 sub to_string {
     my $self = shift;
     my $ret = '';
@@ -531,7 +952,7 @@ sub to_string {
     if ($self->constraints) {
         $ret .= ', CONSTRAINTS: [';
         for my $con ($self->constraints) {
-            $ret .= '[' . $con->to_string . '],';
+            $ret .= '<' . $con->to_string . '>,';
         }
         $ret .= ']';
     }
@@ -560,7 +981,7 @@ sub _validate {    # called internally, obeys is_validating
 
     #   push @errs, $self->validate_logic;
     @errs = grep { $_ } @errs;
-    croak join( '', @errs ) if @errs;
+    confess join( '', @errs ) if @errs;
 }
 
 before _validate => sub {
